/*****************************************************************************
* File Name: main.c
*
* Version: 1.10
*
* Description: This code example demonstrates how to use Watchdog Timer (WDT)
* to reset the device and to periodically generate interrupts. It also shows
* how to use the ILO compensate APIs to improve the WDT interrupt period 
* accuracy.
*
* Related Document: Code example CE210192_PSoC4S_WatchDogTimer.pdf
*
* Hardware Dependency: See code example CE210192_PSoC4S_WatchDogTimer.pdf
*
*******************************************************************************
* Copyright (2016-2020), Cypress Semiconductor Corporation. All rights reserved.
*******************************************************************************
* This software, including source code, documentation and related materials
* (“Software”), is owned by Cypress Semiconductor Corporation or one of its
* subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
* protection (United States and foreign), United States copyright laws and
* international treaty provisions. Therefore, you may use this Software only
* as provided in the license agreement accompanying the software package from
* which you obtained this Software (“EULA”).
*
* If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
* non-transferable license to copy, modify, and compile the Software source
* code solely for use in connection with Cypress’s integrated circuit products.
* Any reproduction, modification, translation, compilation, or representation
* of this Software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
* reserves the right to make changes to the Software without notice. Cypress
* does not assume any liability arising out of the application or use of the
* Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use in any products where a malfunction or
* failure of the Cypress product may reasonably be expected to result in
* significant property damage, injury or death (“High Risk Product”). By
* including Cypress’s product in a High Risk Product, the manufacturer of such
* system or application assumes all risk of such use and in doing so agrees to
* indemnify Cypress against all liability. 
*******************************************************************************/

#include <project.h>
#include "stdio.h"

/* LED States */
#define ON                      0u
#define OFF                     1u

#define RESET_FLAG_ACTIVE       1u
#define SET                     1u
#define CLEAR                   0u

#define WDT_RESET_DEMO         0U

/* Define desired interrupt delay in milliseconds to calculate ilo match count */
#define WAKEUP_PERIOD_IN_MS        500u

/* Define desired delay in microseconds */
#define DESIRED_DELAY           500000u

/* Calculate ILO match count to get wdt periodic updates */
#define ILO_MATCH_COUNTS        ((WAKEUP_PERIOD_IN_MS * CY_SYS_CLK_ILO_DESIRED_FREQ_HZ)/ 1000)

uint32  resetFlag;
uint32  interruptFlag;
uint32  iloMatchCounts = ILO_MATCH_COUNTS;

/* The prototype of the WDT ISR */
CY_ISR_PROTO(WdtIsr);

/* For GCC compiler revise _write() function for printf functionality */
int _write(int file, char *ptr, int len)
{
    (void) file;
int i;
for (i = 0; i < len; i++)
{
UART_UartPutChar(*ptr++);
}
return(len);
}

uint8_t count = 0;

int main()
{
    uint32  tempIloCounts = 0u;
    
    UART_Start();
        /* Enabling global interrupts */
    CyGlobalIntEnable;
    
    /* Turn on Red LED for 2 seconds if reset caused by WDT */
    if (CY_SYS_RESET_WDT == CySysGetResetReason(CY_SYS_RESET_WDT))
    {
        /* Update flag in order to clear WDT */
           resetFlag = RESET_FLAG_ACTIVE;
     //   printf("WDT Reset\n\r");
        
        /* Turn off Blue LED */
           LED_Reset_Write(OFF);
        
        /* Toggle with Red LED to indicate WDT reset */
           LED_WdtReset_Write(ON);
           CyDelay(2000u);
           LED_WdtReset_Write(OFF);
        
        /* Make sure that interrupt is forwarded to the CPU */
            CySysWdtUnmaskInterrupt();
    }
    else
    {
        /* Turn on Blue LED */
        LED_Reset_Write(ON);
        // printf("Power/ XRES Reset\n\r");
        
        /* Configure WDT to be 15-bit wraparound up-counter - 1 MSB is ignored */
        CySysWdtSetIgnoreBits(1);
    } 
    
    CySysWdtClearInterrupt();
   
    /* Configure WDT ISR */
    WdtIsr_StartEx(WdtIsr);
    
    /* Start the WDT */
    CySysWdtEnable();
    CySysWdtUnmaskInterrupt();

    /* Starts the ILO accuracy/Trim measurement */
    CySysClkIloStartMeasurement();
   

    
    for(;;)
    {
        /* If the interrupt has occurred in the ISR */
        if(SET == interruptFlag)
        {
            #if WDT_RESET_DEMO
            count++;
            if(count == 5)
            {
                while(1);
            }
            #endif
          //  printf("Clear interrupt\n\r");
        /* Clear the watchdog interrupt */
           CySysWdtClearInterrupt();
        
        /* Enable WDT interrupt */
           CySysWdtUnmaskInterrupt();
        
        /* WDT match value is updated in order to obtain periodic interrupts */
           CySysWdtWriteMatch(CySysWdtReadMatch() + iloMatchCounts);
        
        /* Clear the software interrupt flag */
           interruptFlag = CLEAR;
        }
        /* Get the ILO compensated counts i.e. the actual counts for the desired ILO frequency 
            Trimming is done to improve ILO accuracy using IMO; ILO default accuracy is +/- 60% */
        if(CYRET_SUCCESS == CySysClkIloCompensate(DESIRED_DELAY, &tempIloCounts))
        {
            iloMatchCounts = (uint32)tempIloCounts;

            /* Once the counts are ready - stop the ILO measurement and put the device to deepsleep */
            CySysClkIloStopMeasurement();
                
            if(CLEAR == interruptFlag)
            {
                /* Device will wakeup through WDT ISR */
                CySysPmDeepSleep();
            }    
            /* Start the ILO measurement after wake up */
               CySysClkIloStartMeasurement();
        }
    }
}

CY_ISR(WdtIsr)
{
        LED_WdtInt_Write(!(LED_WdtInt_Read()));

        /* To make sure WDT ISR does not trigger again till the WDT interrupt is 
           cleared in main(). This is done to make sure WDT clear (for WDT reset)
           happen in main() and not in ISR. */
        /* Disable WDT interrupt */
           CySysWdtMaskInterrupt();
        
        /* Update the flag inorder to clear WDT interrupt in while (1) */
           interruptFlag = SET;
}
/* [] END OF FILE */
